/* Copyright (c) 2010 Marcin Zajaczkowski
 * All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.sf.appinfo.reader;

import net.sf.appinfo.exception.ReaderInitializationException;

import java.io.IOException;
import java.util.jar.Manifest;

/**
 * Abstract ManifestReader implementation with common mechanisms (like synchronization and exception conversion)
 * delegating Manifest initialization to concrete subclasses.
 *
 * @since 0.5.2
 *
 * @author Marcin Zajączkowski, 2010-12-21
 */
public abstract class AbstractManifestReader implements ManifestReader {

    private Manifest manifest;

    /** {@inheritDoc} */
    public Manifest getManifest() throws ReaderInitializationException {
        try {
            return internalGetManifest();
        } catch (IOException e) {
            throw new ReaderInitializationException(e);
        }
    }

    private synchronized Manifest internalGetManifest() throws ReaderInitializationException, IOException {
        if (manifest == null) {
            manifest = initializeAndReturnManifest();
        }
        return manifest;
    }

    /**
     * Delegated initialization (and return of) Manifest object.
     *
     * Should be implemented in concreate classes which create Manifest from data in file, JAR, etc.
     *
     * Throwing IOException isn't very pretty, but it allows to remove duplication and move conversion IOException to
     * ReaderInitializationException to a superclass. It's only internal method in ManifestReader hierarchy and
     * doesn't complicate cooperation with other classes.
     *
     * @return initialized Manifest object
     * @throws IOException error during Manifest initialization
     * @throws ReaderInitializationException other (no IO) error during Manifest initialization
     */
    protected abstract Manifest initializeAndReturnManifest() throws ReaderInitializationException, IOException;
}
